require_relative 'spec_helper'

describe RangeHeaderParser do
  it 'parses the various versions correctly and splices overlapping/adjacent byte ranges' do
    size = 10000
    expect(described_class.parse('bytes=0-499', 10000)).to eq([0..499])
    expect(described_class.parse('bytes=500-999', 10000)).to eq([500..999])
      
    expect(described_class.parse('bytes=-500', 10000)).to eq([9500..9999])
    expect(described_class.parse('bytes=9500-', 10000)).to eq([9500..9999])
    
    expect(described_class.parse('bytes=500-520,601-999', 10000)).to eq([500..520, 601..999])
    
    expect(described_class.parse('bytes=500-600,601-999', 10000)).to eq([500..999])
    expect(described_class.parse('bytes=601-999,500-600', 10000)).to eq([500..999])
    expect(described_class.parse('bytes=500-700,601-999', 10000)).to eq([500..999])
    
    expect(described_class.parse('bytes=500-600,1000-1024,601-999', 10000)).to eq([500..1024])
  end
  
  it 'bails when encountering nonsensical ranges' do
    expect {
      described_class.parse('bytes=-1--13', 1)
    }.to raise_error(described_class::InvalidRange, 'Invalid range spec -1--13 (unparseable)')
    
    expect {
      described_class.parse('bytes=1-13', 1)
    }.to raise_error(described_class::InvalidRange, 'Invalid range spec  1-13 (starts too far in the resource)')
    
    expect {
      described_class.parse('bytes=-999999', 1)
    }.to raise_error(described_class::InvalidRange, 'Invalid range spec -999999 (resource too small)')
    
    expect {
      described_class.parse('bytes=12345-12', 999999)
    }.to raise_error(described_class::InvalidRange, 'Invalid range spec 12345-12 (start and end are flipped)')
  end
  
  it 'handles examples from the RFC 2616' do
    # https://www.w3.org/Protocols/rfc2616/rfc2616-sec14.html#sec14.35.1
    # Examples of byte-ranges-specifier values (assuming an entity-body of length 10000):
    size = 10000
    
    # The first 500 bytes (byte offsets 0-499, inclusive):
    parsed_ranges = described_class.parse('bytes=0-499', size)
    expect(parsed_ranges).to be_one
    range = parsed_ranges.first
    expect(range.to_a.length).to eq(500)
    expect(range).to eq(0..499)
    
    # The second 500 bytes (byte offsets 500-999, inclusive):
    parsed_ranges = described_class.parse('bytes=500-999', size)
    expect(parsed_ranges).to be_one
    range = parsed_ranges.first
    expect(range.to_a.length).to eq(500)
    expect(range).to eq(500..999)
    
    # The final 500 bytes (byte offsets 9500-9999, inclusive):
    range = described_class.parse('bytes=-500', size).first
    expect(range.to_a.length).to eq(500)
    expect(range).to eq(9500..9999)
    range = described_class.parse('bytes=9500-', size).first
    expect(range.to_a.length).to eq(500)
    expect(range).to eq(9500..9999)
    
    # The first and last bytes only (bytes 0 and 9999):
    a, b = described_class.parse('bytes=0-0,-1', size)
    expect(a).to eq(0..0)
    expect(b).to eq(9999..9999)
    
    # Several legal but not canonical specifications of the second 500
    # bytes (byte offsets 500-999, inclusive):
    %w( bytes=500-600,601-999 bytes=500-700,601-999).each do |bytespec|
      ranges = described_class.parse(bytespec, size)
      expect(ranges).to be_one
      range = ranges.first
      expect(range).to eq(500..999)
    end
  end
  
  it 'clamps ranges' do
    size = 100
    expect(described_class.parse('bytes=0-499', size)).to eq([0..99])
  end
  
  it 'bails on invalid headers' do
    %w(
      tingies=100-200
      100-200
      not-a-header-value-at-all
    ).each do | garbage |
      expect {
        described_class.parse(garbage, 10000)
      }.to raise_error(described_class::InvalidHeader, /#{Regexp.escape(garbage)}/)
    end
  end
  
  it 'bails on negative ranges' do
    %w(
      bytes=3-0
    ).each do | garbage |
      expect {
        described_class.parse(garbage, 10000)
      }.to raise_error(described_class::InvalidRange)
    end
  end
  
  it 'bails on invalid ranges' do
    %w(
      bytes=-
      bytes=more
      bytes=more-and-more
    ).each do | garbage |
      expect {
        described_class.parse(garbage, 10000)
      }.to raise_error(described_class::InvalidRange)
    end
  end
end
